/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <cstdio>
#include <cctype>
#include <vector>
#include <map>

using namespace std;

#include "common.h"
#include "field/primal.h"
#include "poly/polynomial.h"
#include "factor/factor.h"

CONSTRUCT_FFPRIMAL(teleso,1);
CONSTRUCT_POLYNOMIAL(polynom,teleso,1);
namespace poly_fft {
  #define FFTMUL
  #undef POLYNOMIAL_H_BY_MILAN_STRAKA_2006 //this doesn't feel right, but have no better idea...
  #undef POLYNOMIAL_COMMON_H_BY_MILAN_STRAKA_2006  
  #undef POLYNOMIAL_ALLOCATOR_H_BY_MILAN_STRAKA_2006
  #include "poly/polynomial.h"
  CONSTRUCT_POLYNOMIAL(polynom,teleso,2);
  #undef FFTMUL
};

template <class T, class P>
void work(unsigned bits, unsigned deg, unsigned num, const char *name) {
  DEFINE_STRUCTURES_TEMPLATE(P);

  clock_t start,sum=0;
  T::initRandom(bits);
  if (!P::init(deg)) return;

  P f(deg);
  pole p;
  for (unsigned i=0;i<num;i++) {
    f.random(deg);

    start=clock();
    factoring<T,P>::factor(f,p);
    sum+=clock()-start;

    if (!factoring<T,P>::test(f,p)) {puts("Error when factoring"); exit(0); }
  }

  if (num>1)  
    printf("%s%u random tests were solved in %g seconds, that is %g seconds for one test.\n",
	   name, num, sum / (double) CLOCKS_PER_SEC, sum / (double) CLOCKS_PER_SEC / (double) num);
  else
    printf("%sRandom test was solved in %g seconds.\n", name, sum / (double) CLOCKS_PER_SEC);

  P::destroy();
}


int main(int argc,char *argv[]) {  
  if (argc < 3) {puts("Usage: bench field_bits degree [number_of_tests]"); return 0;}
  unsigned b,d,n=1;
  if (!parse_unsigned(argv[1],&b)) STOP("Error parsing field_bits '%s'!\n",argv[1]);
  if (!parse_unsigned(argv[2],&d)) STOP("Error parsing degree '%s'!\n",argv[2]);
  if (argc>=4 && !parse_unsigned(argv[3],&n)) STOP("Error parsing number_of_tests '%s'!\n",argv[3]);
  verbose=0; // during development verbose is on by default... but here it should be off

  srand(time(NULL));
  work<teleso, polynom>(b,d,n,"");
  work<teleso, poly_fft::polynom>(b,d,n,"FFTMUL: ");

  return 0;
}
