/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef FACTOR_H_BY_MILAN_STRAKA_INCLUDING

class dd {
 public:
  static void factor(P& f, pole& p);
};

#endif

#ifdef FACTOR_H_BY_MILAN_STRAKA_DECLARING

template<class T, class P>
void factoring<T,P>::dd::factor(P& f, pole& p)  {
  LOG("DD(%d) entering.\n",f.deg());
  if (f.deg()<2/*==1*/) {p.push_back(prvek(1,f)); return; }

  unsigned l=usqrt(f.deg()/2);
  P tmp(f.deg()),tmp2(f.deg());
  typename P::inverse finv; finv=P::invert(f, 2*f.deg());

  //1) h[0..l-1] h[i]=x^q^i % f
  LOG("DD [1/5] computing h...\n");
  vector<P> h;
  h.push_back(P::pex().copy());
  tmp=(h[0] ^ T::fsize()) % finv; h.push_back(tmp.copy());
  LOGMORE("[1/%d] [2/%d] ", l, l);
  for (unsigned i=2;i<l;i++) {tmp=P::compose(h[i-1],h[1],finv); h.push_back(tmp.copy()); LOGMORE("[%d/%d] ", i+1, l);}
  LOGMORE("\n");

  LOG("DD [2/5] computing H...\n");
  //2) H[1..l], H[i]=x^q^lj % f
  vector<P> H;
  H.push_back(P::pzero());
  tmp=P::compose(h[l-1],h[1],finv); H.push_back(tmp.copy());  
  LOGMORE("[%d/%d] ", 1, l);
  for (unsigned i=2;i<=l;i++) {tmp=P::compose(H[i-1],H[1],finv); H.push_back(tmp.copy()); LOGMORE("[%d/%d] ", i, l);}
  LOGMORE("\n");

  LOG("DD [3/5] computing I...\n");
  //3) I[1..l], I[j]=\prod_i=0^{l-1} (H_j - h_i) % f
  vector<P> I;
  I.push_back(P::pzero());
  for (unsigned j=1;j<=l;j++) {
    tmp=P::pone().clone();
    for (unsigned i=0;i<l;i++) { tmp2 = H[j] - h[i]; tmp=tmp*tmp2 % finv; }
    I.push_back(tmp.copy());
  }

  LOG("DD [4/5] computing F...\n");
  //4) F[1..l], F[j]=\gcd(f,I_j)
  vector<P> F;
  F.push_back(P::pzero());
  for (unsigned j=1;j<=l;j++) {
    tmp=P::gcd(f,I[j]);
    if (tmp.deg()) { finv=P::invert(tmp,f.deg()); f=f/finv; }
    F.push_back(tmp.copy());
  }

  LOG("DD [5/5] computing results...\n");
  //5)
  for (unsigned j=1;j<=l;j++) {
    unsigned i=l-1; do {
      tmp2 = H[j] - h[i];
      tmp = P::gcd(F[j],tmp2);
      if (tmp.deg()) {
	finv=P::invert(tmp,F[j].deg()); F[j]=F[j]/finv;
	p.push_back(prvek(j*l-i,tmp.copy()));
      }
    } while (i--);
  }
  
  //6)
  if (f.deg()) p.push_back(prvek(f.deg(),f));
}

#endif
