/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef FACTOR_H_BY_MILAN_STRAKA_INCLUDING

class ed {
 public:  
  static void factor(P& f, pole& p, unsigned d);
  static void sumaMocnin(const P& a, const P& x, const typename P::inverse& f, unsigned k, P& res);
private:
  static void factor_real(P& f, pole& p, unsigned d);
};

#endif


#ifdef FACTOR_H_BY_MILAN_STRAKA_DECLARING

template<class T, class P>
void factoring<T,P>::ed::factor(P& f, pole& p, unsigned d) {
  LOG("ED(%d of deg %d) entering.\n",f.deg()/d,d);

  if (f.deg()/d > 1) p.reserve(f.deg()/d);
  factor_real(f, p, d);
  if (p.size() >= 500) LOGMORE("\n");
}

template<class T, class P>
void factoring<T,P>::ed::factor_real(P& f, pole& p, unsigned d) {
  if (f.deg()==d) {p.push_back(prvek(1,f)); if (p.size()%500==0) LOGMORE("[%d/%d] ", p.size(), p.capacity()); return; }

  P a(f.deg()), b(f.deg()), c(f.deg());
  bool char_not_two=T::fchar().bitTest(0)!=0;
  T mocnina; 
  if (char_not_two) {T inv=T::invert(T::fone()+T::fone()); mocnina=T::fchar() - T::fone(); mocnina=mocnina*inv;}

  typename P::inverse finv; finv=P::invert(f,2*f.deg());
  P x(f.deg());
  x=(P::pex() ^ T::fchar()) % finv;

  do {
    a.random(f.deg()-1);
    sumaMocnin(a, x, finv, T::fext()*d, b);
    if (char_not_two) {b=(b ^ mocnina) % finv;b=b-P::pone();}
    c=P::gcd(b,f);
    if (char_not_two && (c.deg()==0 || c.deg()==f.deg())) {
      b=b+P::pone();  b=b+P::pone();
      c=P::gcd(b,f);
    }
  } while (c.deg()==0 || c.deg()==f.deg());

  finv=P::invert(c,f.deg());
  f=f/finv;
  factor_real(c,p,d);
  factor_real(f,p,d);
}

template<class T, class P>
void factoring<T,P>::ed::sumaMocnin(const P& a, const P& x, const typename P::inverse& f, unsigned k, P& A) {
  P W(f.deg()), X(f.deg()), S(f.deg()), tmp(f.deg());

  X=x.clone(); S=a.clone();
  if (k&1) { A=a.clone(); W=X.clone(); }
  else { A=P::pzero().clone(); W=P::pex().clone(); }

  while(k>>=1,k) {
    tmp=P::compose(S,X,f); S=S+tmp;
    X=P::compose(X,X,f);
    if (k&1) {
      tmp=P::compose(S,W,f); A=A+tmp;
      W=P::compose(W,X,f);
    }
  }
}


#endif
