/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef FACTOR_H_BY_MILAN_STRAKA_2006
#define FACTOR_H_BY_MILAN_STRAKA_2006

#include "../common.h"
#include <vector>
#include <map>
#include <algorithm>

template<class T, class P>
class factoring {
 private:
  DEFINE_STRUCTURES_TEMPLATE(P);
 public:

  #define FACTOR_H_BY_MILAN_STRAKA_INCLUDING
  #include "sffactor.h"
  #include "ddfactor.h"
  #include "edfactor.h"
  #undef FACTOR_H_BY_MILAN_STRAKA_INCLUDING

  static void factor(const P& f, pole& p);
  template<class S> static void collect(const S& what, P& where, unsigned maxdeg);
  static bool test(const P& f, const pole& p);
};

template<class T, class P>
void factoring<T,P>::factor(const P& f, pole& p) {
  if (!f.deg()) {p.push_back(prvek(1,f));return;}
  p.clear();

  //leading coefficient
  P myf(f.deg(),f);
  P lc=myf.leadingCoef();
  if (lc!=P::pone()) p.push_back(prvek(1,lc));
  myf.monify();

  //lets begin
  mapa m;
  sf::factor(myf,m);
  for (mapa_it it=m.begin();it!=m.end();it++) {
    pole g;
    dd::factor(it->second, g);
    for (pole_it git=g.begin(); git!=g.end(); git++) {
      pole h;
      ed::factor(git->second, h, git->first);
      sort(h.begin(),h.end(),typename P::compare_pairs());
      for (pole_it hit=h.begin(); hit!=h.end(); hit++)
	p.push_back(prvek(it->first, hit->second));
    }
  }
}

template<class T, class P> template<class S> 
inline void factoring<T,P>::collect(const S& what, P& where, unsigned maxdeg) {
  where=P::pone().clone();
  P tmp(maxdeg);
  for (typename S::const_iterator it=what.begin(); it!=what.end(); it++) {
    tmp=it->second ^ T::fromUnsigned(it->first);
    where=where*tmp;
  }
}

template<class T, class P> 
bool factoring<T,P>::test(const P& f, const pole& p) {
  P tmp(f.deg());
  collect(p, tmp, f.deg());
  return f==tmp;
}

#define FACTOR_H_BY_MILAN_STRAKA_DECLARING
#include "sffactor.h"
#include "ddfactor.h"
#include "edfactor.h"
#undef FACTOR_H_BY_MILAN_STRAKA_DECLARING


#endif
