/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef FACTOR_H_BY_MILAN_STRAKA_INCLUDING

class sf {
 public:
  static void factor(P& f, mapa& m);
};

#endif

#ifdef FACTOR_H_BY_MILAN_STRAKA_DECLARING

template<class T, class P>
void factoring<T,P>::sf::factor(P& f, mapa& m)  {
  LOG("SF(%d) entering.\n",f.deg());
  if (!f.deg()) {m.insert(prvek(1,f)); return; }
  P der(f.deg()), c(f.deg()), v(f.deg()), w(f.deg()), g(f.deg()); typename P::inverse inv;
  der=P::derivate(f); c=P::gcd(f,der); inv=P::invert(c,f.deg());
  v=f/inv; w=der/inv;
  unsigned i=0;
  while (v.deg()) {
    i++;
    der=P::derivate(v); w=w-der; g=P::gcd(v,w);
    if (g.deg()) m.insert(prvek(i,g.copy()));
    inv=P::invert(g,(v.deg() > w.deg()) ? v.deg() : w.deg());
    v=v/inv; w=w/inv;
  }
  collect(m,g,f.deg()); inv=P::invert(g,f.deg());
  c=f/inv;
  if (!c.deg()) return;

  //bad case, recursion
  LOG("SF(%d) recursion.\n",f.deg());
  unsigned p=T::fchar().toUnsigned(),zmax=0;
  c.pthroot(p); //c=c^1/p
  mapa gg,zz;
  gg.swap(m);
  factor(c,zz);

  //and compose m from gg and zz
  LOG("SF(%d) combining results.\n",f.deg());
  for (mapa_it zit=zz.begin();zit!=zz.end();zit++)
    for (mapa_it git=gg.begin();git!=gg.end();git++) {
      g=P::gcd(git->second,zit->second);
      if (g.deg()) m.insert(prvek(zit->first*p+git->first,g.copy()));
    }
  for (mapa_it zit=zz.begin();zit!=zz.end();zit++) {
    g=P::pone().clone();
    unsigned i=zit->first*p;
    for (unsigned j=1;j<p;j++) if (m.count(j+i)) g=g*m[j+i];
    if (g.deg()) {
      inv=P::invert(g,zit->second.deg());
      zit->second=zit->second/inv;
    }
    if (zit->second.deg()) m.insert(prvek(i,zit->second));
    if (zit->first > zmax) zmax=zit->first;
  }
  for (mapa_it git=gg.begin();git!=gg.end();git++) {
    g=P::pone().clone();
    unsigned i=git->first;
    for (unsigned j=1;j<=zmax;j++) if (m.count(j*p+i)) g=g*m[j*p+i];
    if (g.deg()) {
      inv=P::invert(g,git->second.deg());
      git->second=git->second/inv;
    }
    if (git->second.deg()) m.insert(prvek(git->first,git->second));
  }
}

#endif
