/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <cstdio>
#include <cctype>
#include <vector>
#include <map>

using namespace std;

#include "common.h"
#include "field/primal.h"
#include "poly/polynomial.h"
#include "factor/factor.h"

CONSTRUCT_FFPRIMAL(teleso,1);
CONSTRUCT_POLYNOMIAL(polynom,teleso,1);
namespace poly_fft {
  #define FFTMUL
  #undef POLYNOMIAL_H_BY_MILAN_STRAKA_2006 //this doesn't feel right, but have no better idea...
  #undef POLYNOMIAL_COMMON_H_BY_MILAN_STRAKA_2006  
  #undef POLYNOMIAL_ALLOCATOR_H_BY_MILAN_STRAKA_2006
  #include "poly/polynomial.h"
  CONSTRUCT_POLYNOMIAL(polynom,teleso,2);
  #undef FFTMUL
};

int have_gf=0;
int factored_anything=0;
int fft=0;

void print_help(void) {
  puts("Usage: fapof [OPTIONS] [GFp] [POLYNOMIALS]");
  puts("Print the ireducibile factors of each polynom over finite field GF(p).");
  puts("If no input is specified on the command line, it is read from standart input.");
  puts("");
  puts("Options: -v      be verbose");
  puts("         -vv     be more verbose in DD and ED phase");
  puts("         --fft   use multiplication based on FFT. Quicker but needs smaller GF.");
  puts("         --help  prints this help");
  puts("");
  puts("Example: fapof GF3 x-3 GF5 x^2+2x+1");
  exit(0);
}

void use_option(char *str) {
  if (!strcmp(str,"-v")) verbose=1;
  else if (!strcmp(str,"-vv")) verbose=2;
  else if (!strcmp(str,"--fft")) fft=1;
  else if (!strcmp(str,"--help")) print_help();
  else printf("Unknown option %s.\nTry 'fapof --help' for more information.\n",str);
}

void set_gf(char *str) {
  if (!teleso::initFromStr(str)) return;
  LOG("Using GF(%s)\n",str);
  have_gf=1;
}

template<class P>
void factor(char *str,unsigned maxdeg) {
  DEFINE_STRUCTURES_TEMPLATE(P);
  if (!have_gf) {ERROR("Please specify a finite field before factoring a polynomial!\n");return;}
  if (!P::init(maxdeg)) return;
  {
    P f(maxdeg);
    if (f.read(str,maxdeg)) {
      pole p;
      factoring<teleso, P>::factor(f,p);
      for (pole_it it=p.begin();it!=p.end();it++) {
	if (it!=p.begin()) putchar(' ');
	if (it->second.deg()==0) it->second.write();
	else {putchar('(');it->second.write();putchar(')');}
	if (it->first>1) printf("^%u", it->first);
      }
      putchar('\n');
      factored_anything=1;
    }
  }
  P::destroy();
}

void process_line(char *str) {
  while (isspace(*str)) str++;
  if (tolower(str[0])=='h' && tolower(str[1])=='e' && tolower(str[2])=='l' && tolower(str[3])=='p') {
    str+=4;
    while (isspace(*str)) str++;
    puts("Commands:");
    puts("  gf [number]: use finite field with [number] elements");
    puts("  [polynomial]: factor the given polynomial");
    puts("The polynomials can be written like this: 3x^4 + 2 x ^ 5 + 3 - 1 x");
  }
  while (isspace(*str)) str++;
  if (tolower(str[0])=='g' && tolower(str[1])=='f') {
    int i=2;
    while (isspace(str[i])) i++;
    char *gf_start=str+i;
    while (str[i] && !isspace(str[i])) i++;
    if (str[i]) str[i++]='\0';
    str+=i;

    set_gf(gf_start);
  }
  while (isspace(*str)) str++;
  if (!*str) return;

  //now we have some input... guess maximum degree
  unsigned deg=0,d;
  if (strpbrk(str,"xX")) deg=1;
  for (char *ptr=strchr(str,'^');ptr;ptr=strchr(ptr+1,'^'))
    if (parse_unsigned(ptr+1,&d) && d>deg) deg=d;

  if (!fft) factor<polynom>(str,deg);
  else factor<poly_fft::polynom>(str,deg);
}

int main(int argc,char *argv[]) {
  int arg=1;
  while (arg < argc && *argv[arg]=='-') use_option(argv[arg++]);
  srand(time(NULL));

  //now lets process arguments
  while (arg<argc) process_line(argv[arg++]);
  if (factored_anything) return 0;

  //ok, stdin
  int c;
  vector<char> v; // too lazy to double by realloc

  do {
    while (c=getchar(),c!=EOF && c!='\n') v.push_back(c);
    v.push_back('\0');
    process_line(&v[0]); // this is according to stl
    v.clear();
  } while (c!=EOF);

  return 0;
}
