/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#include <cstdio>
#include <cctype>
#include <vector>

#define MAX_ZNAM 128
#define MAX_POLY 128

using namespace std;

#include "common.h"
#include "field/primal.h"
#include "poly/polynomial.h"

CONSTRUCT_FFPRIMAL(teleso,1);
CONSTRUCT_POLYNOMIAL(polynom,teleso,1);

unsigned p,maxdeg;

int argc;
int arg;
char **argv;

unsigned zznam[MAX_ZNAM];
int zznam_len;

polynom* zpoly[MAX_ZNAM];
int zpoly_len;

unsigned next_item() {
  unsigned par,ret;

  if (arg==argc) return 0U;
  switch(argv[arg][0]) {
  case '+':
  case '-':
  case '/':
  case '%':
  case 'g':
  case 'c':
  case '[':
  case ']':
    return argv[arg++][0];   
  case '.':
    return (argv[arg++][1]=='%') ? 'm' : '.';
  case 'i':
    if (argv[arg][1]) sscanf(argv[arg]+1,"%u",&par);
    else sscanf(argv[++arg],"%u",&par);
    arg++;
    return 'i'+(par<<8);
  case 'r':
    if (argv[arg][1]) sscanf(argv[arg]+1,"%u",&par);
    else sscanf(argv[++arg],"%u",&par);
    arg++;
    return 'r'+(par<<8);
  case '^':
    ret='^';if (argv[arg][1]=='%') {ret='e';argv[arg]++;}
    if (argv[arg][1]) sscanf(argv[arg]+1,"%u",&par);
    else sscanf(argv[++arg],"%u",&par);
    arg++;
    return ret+(par<<8);
  }
  zpoly[zpoly_len]=new polynom(maxdeg);
  zpoly[zpoly_len++]->read(argv[arg],maxdeg);
  arg++;
  return 1U;
}

void apply() {
  switch (zznam[--zznam_len] & 0xff) {
  case '+': zpoly_len--;*zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] + *zpoly[zpoly_len]; 
    delete zpoly[zpoly_len]; break;
  case '-': zpoly_len--;*zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] - *zpoly[zpoly_len];
    delete zpoly[zpoly_len]; break;
  case '.': zpoly_len--;*zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] * *zpoly[zpoly_len];
    delete zpoly[zpoly_len]; break;
  case 'm': 
    zpoly_len-=2;
    {
      polynom::inverse b;b=polynom::invert(*zpoly[zpoly_len+1],2*zpoly[zpoly_len+1]->deg());
      *zpoly[zpoly_len-1]= (*zpoly[zpoly_len-1] * *zpoly[zpoly_len]) % b;
      delete zpoly[zpoly_len+1]; delete zpoly[zpoly_len];
    }
    break;
  case '^': *zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] ^ teleso::fromUnsigned(zznam[zznam_len]>>8); break;
  case 'e': 
    zpoly_len--;
    {
      polynom::inverse b;b=polynom::invert(*zpoly[zpoly_len],2*zpoly[zpoly_len]->deg());
      *zpoly[zpoly_len-1]= (*zpoly[zpoly_len-1] ^ teleso::fromUnsigned(zznam[zznam_len]>>8)) % b;
      delete zpoly[zpoly_len];
    }
    break;
  case 'i':
    {
      polynom::inverse b;b=polynom::invert(*zpoly[zpoly_len-1],(zznam[zznam_len]>>8));
      *zpoly[zpoly_len-1]=b;
    }    
    break;
  case 'g':zpoly_len--;*zpoly[zpoly_len-1]=polynom::gcd(*zpoly[zpoly_len-1],*zpoly[zpoly_len]);
    delete zpoly[zpoly_len]; break;
    break;
  case 'c': 
    zpoly_len-=2;
    {
      polynom::inverse b;b=polynom::invert(*zpoly[zpoly_len+1],2*zpoly[zpoly_len+1]->deg());
      *zpoly[zpoly_len-1]= polynom::compose(*zpoly[zpoly_len-1],*zpoly[zpoly_len],b);
      delete zpoly[zpoly_len+1]; delete zpoly[zpoly_len];
    }
    break;
  case '/':
  case '%': 
    zpoly_len--;
    {
      polynom::inverse b;b=polynom::invert(*zpoly[zpoly_len],2*zpoly[zpoly_len]->deg());
      if (zznam[zznam_len]=='/') *zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] / b;
      else *zpoly[zpoly_len-1]=*zpoly[zpoly_len-1] % b;
      delete zpoly[zpoly_len];
    }
    break;
  }
}

int is_in(const char c, const char *str) {
  while (*str) if (c==*str++) return 1;
  return 0;
}

int main(int ac,char *av[]) {  
  arg=2;
  argc=ac;
  argv=av;
  if (argc < 2) {puts("Usage: pc prime[,maxdeg=10000] expression");return 0;}
  maxdeg=10000;
  if (strchr(argv[1],',')) {char *tmp=strchr(argv[1],',');*tmp='\0';sscanf(tmp+1,"%u",&maxdeg);}
  if (!teleso::initFromStr(argv[1])) return 0;
  if (!polynom::init(maxdeg)) return 0;
  srand(time(NULL));

  zznam_len=zpoly_len=0;
  unsigned c;
  while (c=next_item(),c!=0U) {
    switch(c&0xff) {
    case '+':
    case '-':
      while (zznam_len && !is_in(zznam[zznam_len-1],"[")) apply();
      zznam[zznam_len++]=c;
      break;
    case '.':
    case 'm':
    case '/':
    case '%':
      while (zznam_len && !is_in(zznam[zznam_len-1],"[+-")) apply();
      zznam[zznam_len++]=c;
      break;
    case '^':
    case 'e':
      while (zznam_len && !is_in(zznam[zznam_len-1],"[+-.m/%")) apply();
      zznam[zznam_len++]=c;
      break;
    case 'i':
      zznam[zznam_len++]=c;
      apply();
      break;
    case 'g':
      zznam[zznam_len++]=c;
      break;
    case 'c':
      zznam[zznam_len++]=c;
      break;
    case 'r':
      zpoly[zpoly_len]=new polynom(maxdeg);
      zpoly[zpoly_len++]->random(c>>8);
    case '[':
      zznam[zznam_len++]='[';
      break;
    case ']':
      while (zznam_len && zznam[zznam_len-1] != ']') apply();
      if (zznam_len) zznam_len--;
      break;
    }
  }

  while (zznam_len) apply();
  if (zpoly_len==1) {zpoly[0]->write();delete zpoly[0];putchar('\n');}
  else puts("Bad syntax!");

  polynom::destroy();

  return 0;
}
