/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_PROTOTYPES
#define define_add_sub_proto(operation,privfunc,operatorname)						\
private:												\
  struct operation##Wrapper {										\
  public:												\
    operation##Wrapper(const polynom& pa,const polynom& pb) : a(pa),b(pb) {}				\
    const polynom &a,&b;										\
  };													\
  static void privfunc(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b);				\
public:													\
  operation##Wrapper operatorname(const polynom& b) const { return operation##Wrapper(*this,b); }	\
  void operator=(const operation##Wrapper& c);

define_add_sub_proto(Add,add,operator+)
define_add_sub_proto(Sub,sub,operator-)
#undef define_add_sub_proto
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS

#define define_add_sub(operation,privfunc,binop,unop,sametest)					\
template<class T, int id>									\
inline void polynomial<T,id>::operator=(const operation##Wrapper& c) {				\
  privfunc(p,c.a.p,c.b.p);									\
}												\
												\
template<class T, int id>									\
void polynomial<T,id>::privfunc(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b) {		\
  unsigned min=a->deg;										\
  if (b->deg == min) { /* we will try to ignore "the same top parts" not to use space! */	\
    while (min && sametest) min--;								\
    to->deg=min;										\
    while (min) to->c[min]=a->c[min] binop b->c[min],min--;					\
    to->c[0]=a->c[0] binop b->c[0];								\
    return;											\
  }												\
  if (b->deg < min) min=b->deg;									\
  for (unsigned i=min;i>0;i--) to->c[i]=a->c[i] binop b->c[i];to->c[0]=a->c[0] binop b->c[0];	\
  for (unsigned i=a->deg;i>min;i--) to->c[i]=a->c[i];						\
  for (unsigned i=b->deg;i>min;i--) to->c[i]=unop b->c[i];					\
  to->deg=(a->deg > b->deg) ? a->deg : b->deg;							\
  DEGRIFY_COEFS(to);										\
}

define_add_sub(Add,add,+,,a->c[min]+b->c[min]==T::fzero())
define_add_sub(Sub,sub,-,T::fzero()-,a->c[min]==b->c[min])
#undef define_add_sub
#endif
