/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifndef POLYNOMIAL_ALLOCATOR_H_BY_MILAN_STRAKA_2006
#define POLYNOMIAL_ALLOCATOR_H_BY_MILAN_STRAKA_2006

#include "../common.h"
#include "common.h"

template<class T,int id>
class alloc {
 public:
  static polynomial_coefs<T>* getmem(unsigned deg);
  static polynomial_coefs<T>* getmem(polynomial_coefs<T> *ptr);
  static void freemem(polynomial_coefs<T>* ptr);
  static void init(unsigned deg);
  static void destroy();
 private:
  static unsigned max_deg;
};

#define CONSTRUCT_ALLOCATOR(T,id) template<> unsigned alloc<T,id>::max_deg=0;

//global allocator creation
 
template<class T,int id>
inline void alloc<T,id>::init(unsigned deg) {
  alloc<T,id>::max_deg=deg;
}

template<class T,int id>
inline void alloc<T,id>::destroy() {}

//allocating functions

template<class T,int id>
inline polynomial_coefs<T>* alloc<T,id>::getmem(unsigned deg) {
  polynomial_coefs<T>* ptr=(polynomial_coefs<T>*) new char[sizeof(polynomial_coefs<T>)+deg*sizeof(T)];
  ptr->ref=1;
  return ptr;
}

template<class T,int id>
inline polynomial_coefs<T>* alloc<T,id>::getmem(polynomial_coefs<T>* ptr) {
  ptr->ref++;
  return ptr;
}

template<class T,int id>
inline void alloc<T,id>::freemem(polynomial_coefs<T>* ptr) {
  if (--ptr->ref==0) delete[] ptr;
}

#endif
