/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_PROTOTYPES
public:
int operator==(const polynom& b) const;
int operator!=(const polynom& b) const;

// this is just for sorting factors when printing output... 
// At first according to degree, then according to coefs from MSC
struct compare_pairs { bool operator()(const pair<unsigned, polynom> a, const pair<unsigned, polynom> b); };
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS
template<class T, int id>
inline int polynomial<T,id>::operator==(const polynom& b) const {
  polynom_coefs *pb=b.p;
  if (p->deg != pb->deg) return 0; 
  for (unsigned i=p->deg;i>0;i--) if (p->c[i] != pb->c[i]) return 0;
  return p->c[0] == pb->c[0];
}

template<class T, int id>
inline int polynomial<T,id>::operator!=(const polynom& b) const {
  return !(*this==b);
}

template<class T, int id>
bool polynomial<T,id>::compare_pairs::operator()(const pair<unsigned, polynom> a, const pair<unsigned, polynom> b) {
  polynom_coefs *pa=a.second.p, *pb=b.second.p;
  if (pa->deg < pb->deg) return true;
  for (unsigned i=pa->deg;i>0;i--) 
    if (pa->c[i] < pb->c[i]) return true;
    else if (pa->c[i] > pb->c[i]) return false;
  return pa->c[0] < pb->c[0];
}

#endif
