/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_PROTOTYPES
private:
  struct ExpWrapper {
  public:
    ExpWrapper(const polynom& pa,const T& pd) : a(pa),d(pd) {}
    typename polynom::ExpModWrapper 
      operator%(const typename polynom::inverse& pi) { return ExpModWrapper(*this,pi); }
    const polynom &a; const T& d;
  };
  struct ExpModWrapper {
  public:
    ExpModWrapper(const ExpWrapper &pe,const inverse& pi) : a(pe.a),d(pe.d),i(pi) {}
    const polynom &a;const T& d;const inverse &i;
  };
  static void exp(polynom_coefs *to, polynom_coefs *orig, const T& ex);
  static void expmod(polynom_coefs *to, polynom_coefs *orig, const T& ex, 
		     polynom_coefs *inv, polynom_coefs *inv_b);

public:
  ExpWrapper operator^(const T& pd) const { return ExpWrapper(*this,pd); }
  void operator=(const ExpWrapper& c);
  void operator=(const ExpModWrapper& c);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DECL
  polynom_coefs *exptmp,*powtmp;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_CREATE
  data.exptmp=Alloc::getmem(2*deg);
  data.powtmp=Alloc::getmem(2*deg);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DESTROY
  if (data.exptmp) Alloc::freemem(data.exptmp); data.exptmp=NULL;
  if (data.powtmp) Alloc::freemem(data.powtmp); data.powtmp=NULL;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS

template<class T, int id>
void polynomial<T,id>::exp(polynom_coefs *to, polynom_coefs *orig, const T& ex) {
  if (ex==T::fzero()) {to->deg=0;to->c[0]=T::fone();return;}
  unsigned d=ex.bitHighest();

  //init x
  polynom_coefs *x=data.exptmp, *res=data.powtmp, *multmp=data.multmp;
  CLONE_COEFS(x,orig);

  //init res
  if (ex.bitTest(0)) CLONE_COEFS(res,orig)
  else {res->deg=0;res->c[0]=T::fone();}

  unsigned b=1;
  while(b<=d) {
    //x=x*x
    mul(multmp,x,x);CLONE_COEFS(x,multmp);
    
    if (ex.bitTest(b)) {
      //res=res*x
      mul(multmp,res,x);CLONE_COEFS(res,multmp);
    }
    b++;
  }

  CLONE_COEFS(to,res);
  DEGRIFY_COEFS(to);
}

template<class T, int id>
inline void polynomial<T,id>::operator=(const ExpWrapper& c) {
  exp(p,c.a.p,c.d);
}

template<class T, int id>
void polynomial<T,id>::expmod(polynom_coefs *to, polynom_coefs *orig, const T& ex, 
			      polynom_coefs *inv, polynom_coefs *inv_b) {
  unsigned d=ex.bitHighest();
  if (ex==T::fzero()) {to->deg=0;to->c[0]=T::fone();return;}

  //init x
  polynom_coefs *x=data.exptmp, *res=data.powtmp, *multmp=data.multmp;
  CLONE_COEFS(x,orig);

  //init res
  if (ex.bitTest(0)) CLONE_COEFS(res,orig)
  else {res->deg=0;res->c[0]=T::fone();}

  unsigned b=1;
  while(b<=d) {
    //x=x*x
    mul(multmp,x,x);
    mod(x,multmp,inv,inv_b);
    
    if (ex.bitTest(b)) {
      //res=res*x
      mul(multmp,res,x);
      mod(res,multmp,inv,inv_b);
    }
    b++;
  }

  CLONE_COEFS(to,res);
  DEGRIFY_COEFS(to);
}

template<class T, int id>
inline void polynomial<T,id>::operator=(const ExpModWrapper& c) {
  expmod(p,c.a.p,c.d,c.i.p,c.i.b);
}

#endif
