/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_PROTOTYPES
private:
  struct GCDWrapper {
  public:
    GCDWrapper(const polynom& pa,const polynom& pb) : a(pa),b(pb) {}
    const polynom &a,&b;
  };
  //be carefull, private version does not 
  static void gcd(polynom_coefs *to, polynom_coefs *pa, polynom_coefs *pb);

public:
  static GCDWrapper gcd(const polynom& a, const polynom& b) { return GCDWrapper(a,b); }
  void operator=(const GCDWrapper& c);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DECL
  polynom_coefs *gcdtmp1,*gcdtmp2;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_CREATE
  data.gcdtmp1=Alloc::getmem(deg); 
  data.gcdtmp2=Alloc::getmem(deg);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DESTROY
  if (data.gcdtmp1) Alloc::freemem(data.gcdtmp1); data.gcdtmp1=NULL;
  if (data.gcdtmp2) Alloc::freemem(data.gcdtmp2); data.gcdtmp2=NULL;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS
template<class T, int id>
void polynomial<T,id>::gcd(polynom_coefs *to, polynom_coefs *pa, polynom_coefs *pb) {
  if (!pa->deg) {
    if (pa->c[0]==T::fzero()) CLONE_COEFS(to, pb) else to->deg=0, to->c[0]=T::fone();
    return;
  }
  if (!pb->deg) {
    if (pb->c[0]==T::fzero()) CLONE_COEFS(to, pa) else to->deg=0, to->c[0]=T::fone();
    return;
  }

  polynom_coefs *a=data.gcdtmp1,*b=data.gcdtmp2,*t;
  if (pa->deg > pb->deg) { CLONE_COEFS(a,pa); CLONE_COEFS(b,pb); } 
  else                   { CLONE_COEFS(a,pb); CLONE_COEFS(b,pa); }

  unsigned i,j;
  while (b->deg) {
    T inverze=a->c[a->deg]*T::invert(b->c[b->deg]);
    for (i=b->deg,j=a->deg;i>0;i--,j--) a->c[j]=a->c[j]-inverze*b->c[i];
    a->c[j]=a->c[j]-inverze*b->c[0];
    while (a->deg && a->c[a->deg]==T::fzero()) a->deg--;
    if (a->deg < b->deg) {t=a;a=b;b=t;}
  }
  //is b nonzero? Then gcd(a,b)=1
  if (b->c[0]!=T::fzero()) to->deg=0, to->c[0]=T::fone();
  //well, gcd(a,b) is a monified
  else {CLONE_COEFS(to, a); MONIFY_COEFS(to); }
}


template<class T, int id>
inline void polynomial<T,id>::operator=(const GCDWrapper& c) {
  gcd(p,c.a.p,c.b.p);
}
#endif
