/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_PROTOTYPES
private:
  struct MulWrapper {
  public:
    MulWrapper(const polynom& pa,const polynom& pb) : a(pa),b(pb) {}
    typename polynom::MulModWrapper 
      operator%(const typename polynom::inverse& pi) { return MulModWrapper(*this,pi); }
    const polynom &a,&b;
  };
  struct MulModWrapper {
  public:
    MulModWrapper(const MulWrapper &pm,const typename polynom::inverse& pi) : a(pm.a),b(pm.b),i(pi) {}
    const polynom &a,&b;const typename polynom::inverse &i;
  };

  static void mul(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b);
  static void mul_force(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b);

public:
  MulWrapper operator*(const polynom& b) const { return MulWrapper(*this,b); }
  void operator=(const MulWrapper& c);
  void operator=(const MulModWrapper& c);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DECL
  polynom_coefs *multmp;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_CREATE
  data.multmp=Alloc::getmem(2*deg);
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DESTROY
  if (data.multmp) Alloc::freemem(data.multmp); data.multmp=NULL;
#endif

// define mul function
#ifdef FFTMUL
#include "mul_fft.h"
#else
#include "mul_rec.h"
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS
template<class T, int id>
inline void polynomial<T,id>::operator=(const MulModWrapper& c) {
  mul(data.powtmp,c.a.p,c.b.p);
  mod(p,data.powtmp,c.i.p,c.i.b);
}

#ifndef BRUTALMUL
template<class T, int id>
void polynomial<T,id>::mul_force(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b) {
  to->deg=a->deg+b->deg;
  for (unsigned i=0;i<=to->deg;i++) to->c[i]=T::fzero();
  for (unsigned i=a->deg;i>0;i--) {
    for (unsigned j=b->deg;j>0;j--)
      to->c[i+j]=to->c[i+j] + a->c[i]*b->c[j];
    to->c[i]=to->c[i] + a->c[i]*b->c[0];
  }
  for (unsigned j=b->deg;j>0;j--)
    to->c[j]=to->c[j] + a->c[0]*b->c[j];
  to->c[0]=to->c[0] + a->c[0]*b->c[0];
}
#else
template<class T, int id>
void polynomial<T,id>::mul_force(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b) {
  T *pa,*pb,*pc=to->c; unsigned d;
  if (a->deg > b->deg) {pa=a->c; pb=b->c; d=a->deg+12*b->deg; }
  else {pa=b->c; pb=a->c; d=12*a->deg+b->deg; }

  switch (d) {
  default:
  case 0: to->deg=0;
    pc[0]=pa[0]*pb[0];
    return;
  case 1: to->deg=1;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    return;
  case 2: to->deg=2;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    return;
  case 3: to->deg=3;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    return;
  case 4: to->deg=4;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    return;
  case 5: to->deg=5;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    return;
  case 6: to->deg=6;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    return;
  case 7: to->deg=7;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    pc[7]=pa[7]*pb[0];
    return;
  case 8: to->deg=8;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    pc[7]=pa[7]*pb[0];
    pc[8]=pa[8]*pb[0];
    return;
  case 9: to->deg=9;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    pc[7]=pa[7]*pb[0];
    pc[8]=pa[8]*pb[0];
    pc[9]=pa[9]*pb[0];
    return;
  case 10: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    pc[7]=pa[7]*pb[0];
    pc[8]=pa[8]*pb[0];
    pc[9]=pa[9]*pb[0];
    pc[10]=pa[10]*pb[0];
    return;
  case 11: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[1]*pb[0];
    pc[2]=pa[2]*pb[0];
    pc[3]=pa[3]*pb[0];
    pc[4]=pa[4]*pb[0];
    pc[5]=pa[5]*pb[0];
    pc[6]=pa[6]*pb[0];
    pc[7]=pa[7]*pb[0];
    pc[8]=pa[8]*pb[0];
    pc[9]=pa[9]*pb[0];
    pc[10]=pa[10]*pb[0];
    pc[11]=pa[11]*pb[0];
    return;
  case 13: to->deg=2;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1];
    return;
  case 14: to->deg=3;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1];
    return;
  case 15: to->deg=4;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1];
    return;
  case 16: to->deg=5;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1];
    return;
  case 17: to->deg=6;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1];
    return;
  case 18: to->deg=7;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1];
    return;
  case 19: to->deg=8;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[7]*pb[1];
    return;
  case 20: to->deg=9;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[8]*pb[1];
    return;
  case 21: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[9]*pb[1];
    return;
  case 22: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[10]*pb[1];
    return;
  case 23: to->deg=12;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[10]*pb[1]+pa[11]*pb[0];
    pc[12]=pa[11]*pb[1];
    return;
  case 26: to->deg=4;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1];
    pc[4]=pa[2]*pb[2];
    return;
  case 27: to->deg=5;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1];
    pc[5]=pa[3]*pb[2];
    return;
  case 28: to->deg=6;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1];
    pc[6]=pa[4]*pb[2];
    return;
  case 29: to->deg=7;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1];
    pc[7]=pa[5]*pb[2];
    return;
  case 30: to->deg=8;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1];
    pc[8]=pa[6]*pb[2];
    return;
  case 31: to->deg=9;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[6]*pb[2]+pa[7]*pb[1];
    pc[9]=pa[7]*pb[2];
    return;
  case 32: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[7]*pb[2]+pa[8]*pb[1];
    pc[10]=pa[8]*pb[2];
    return;
  case 33: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[8]*pb[2]+pa[9]*pb[1];
    pc[11]=pa[9]*pb[2];
    return;
  case 34: to->deg=12;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[9]*pb[2]+pa[10]*pb[1];
    pc[12]=pa[10]*pb[2];
    return;
  case 35: to->deg=13;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[9]*pb[2]+pa[10]*pb[1]+pa[11]*pb[0];
    pc[12]=pa[10]*pb[2]+pa[11]*pb[1];
    pc[13]=pa[11]*pb[2];
    return;
  case 39: to->deg=6;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2];
    pc[6]=pa[3]*pb[3];
    return;
  case 40: to->deg=7;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2];
    pc[7]=pa[4]*pb[3];
    return;
  case 41: to->deg=8;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2];
    pc[8]=pa[5]*pb[3];
    return;
  case 42: to->deg=9;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2];
    pc[9]=pa[6]*pb[3];
    return;
  case 43: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1];
    pc[9]=pa[6]*pb[3]+pa[7]*pb[2];
    pc[10]=pa[7]*pb[3];
    return;
  case 44: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1];
    pc[10]=pa[7]*pb[3]+pa[8]*pb[2];
    pc[11]=pa[8]*pb[3];
    return;
  case 45: to->deg=12;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1];
    pc[11]=pa[8]*pb[3]+pa[9]*pb[2];
    pc[12]=pa[9]*pb[3];
    return;
  case 46: to->deg=13;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1];
    pc[12]=pa[9]*pb[3]+pa[10]*pb[2];
    pc[13]=pa[10]*pb[3];
    return;
  case 47: to->deg=14;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1]+pa[11]*pb[0];
    pc[12]=pa[9]*pb[3]+pa[10]*pb[2]+pa[11]*pb[1];
    pc[13]=pa[10]*pb[3]+pa[11]*pb[2];
    pc[14]=pa[11]*pb[3];
    return;
  case 52: to->deg=8;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3];
    pc[8]=pa[4]*pb[4];
    return;
  case 53: to->deg=9;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3];
    pc[9]=pa[5]*pb[4];
    return;
  case 54: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3];
    pc[10]=pa[6]*pb[4];
    return;
  case 55: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2];
    pc[10]=pa[6]*pb[4]+pa[7]*pb[3];
    pc[11]=pa[7]*pb[4];
    return;
  case 56: to->deg=12;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1];
    pc[10]=pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2];
    pc[11]=pa[7]*pb[4]+pa[8]*pb[3];
    pc[12]=pa[8]*pb[4];
    return;
  case 57: to->deg=13;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1];
    pc[11]=pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2];
    pc[12]=pa[8]*pb[4]+pa[9]*pb[3];
    pc[13]=pa[9]*pb[4];
    return;
  case 58: to->deg=14;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1];
    pc[12]=pa[8]*pb[4]+pa[9]*pb[3]+pa[10]*pb[2];
    pc[13]=pa[9]*pb[4]+pa[10]*pb[3];
    pc[14]=pa[10]*pb[4];
    return;
  case 59: to->deg=15;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1]+pa[11]*pb[0];
    pc[12]=pa[8]*pb[4]+pa[9]*pb[3]+pa[10]*pb[2]+pa[11]*pb[1];
    pc[13]=pa[9]*pb[4]+pa[10]*pb[3]+pa[11]*pb[2];
    pc[14]=pa[10]*pb[4]+pa[11]*pb[3];
    pc[15]=pa[11]*pb[4];
    return;
  case 65: to->deg=10;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4];
    pc[10]=pa[5]*pb[5];
    return;
  case 66: to->deg=11;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4];
    pc[11]=pa[6]*pb[5];
    return;
  case 67: to->deg=12;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4]+pa[7]*pb[3];
    pc[11]=pa[6]*pb[5]+pa[7]*pb[4];
    pc[12]=pa[7]*pb[5];
    return;
  case 68: to->deg=13;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2];
    pc[11]=pa[6]*pb[5]+pa[7]*pb[4]+pa[8]*pb[3];
    pc[12]=pa[7]*pb[5]+pa[8]*pb[4];
    pc[13]=pa[8]*pb[5];
    return;
  case 69: to->deg=14;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1];
    pc[11]=pa[6]*pb[5]+pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2];
    pc[12]=pa[7]*pb[5]+pa[8]*pb[4]+pa[9]*pb[3];
    pc[13]=pa[8]*pb[5]+pa[9]*pb[4];
    pc[14]=pa[9]*pb[5];
    return;
  case 70: to->deg=15;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[6]*pb[5]+pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1];
    pc[12]=pa[7]*pb[5]+pa[8]*pb[4]+pa[9]*pb[3]+pa[10]*pb[2];
    pc[13]=pa[8]*pb[5]+pa[9]*pb[4]+pa[10]*pb[3];
    pc[14]=pa[9]*pb[5]+pa[10]*pb[4];
    pc[15]=pa[10]*pb[5];
    return;
  case 71: to->deg=16;
    pc[0]=pa[0]*pb[0];
    pc[1]=pa[0]*pb[1]+pa[1]*pb[0];
    pc[2]=pa[0]*pb[2]+pa[1]*pb[1]+pa[2]*pb[0];
    pc[3]=pa[0]*pb[3]+pa[1]*pb[2]+pa[2]*pb[1]+pa[3]*pb[0];
    pc[4]=pa[0]*pb[4]+pa[1]*pb[3]+pa[2]*pb[2]+pa[3]*pb[1]+pa[4]*pb[0];
    pc[5]=pa[0]*pb[5]+pa[1]*pb[4]+pa[2]*pb[3]+pa[3]*pb[2]+pa[4]*pb[1]+pa[5]*pb[0];
    pc[6]=pa[1]*pb[5]+pa[2]*pb[4]+pa[3]*pb[3]+pa[4]*pb[2]+pa[5]*pb[1]+pa[6]*pb[0];
    pc[7]=pa[2]*pb[5]+pa[3]*pb[4]+pa[4]*pb[3]+pa[5]*pb[2]+pa[6]*pb[1]+pa[7]*pb[0];
    pc[8]=pa[3]*pb[5]+pa[4]*pb[4]+pa[5]*pb[3]+pa[6]*pb[2]+pa[7]*pb[1]+pa[8]*pb[0];
    pc[9]=pa[4]*pb[5]+pa[5]*pb[4]+pa[6]*pb[3]+pa[7]*pb[2]+pa[8]*pb[1]+pa[9]*pb[0];
    pc[10]=pa[5]*pb[5]+pa[6]*pb[4]+pa[7]*pb[3]+pa[8]*pb[2]+pa[9]*pb[1]+pa[10]*pb[0];
    pc[11]=pa[6]*pb[5]+pa[7]*pb[4]+pa[8]*pb[3]+pa[9]*pb[2]+pa[10]*pb[1]+pa[11]*pb[0];
    pc[12]=pa[7]*pb[5]+pa[8]*pb[4]+pa[9]*pb[3]+pa[10]*pb[2]+pa[11]*pb[1];
    pc[13]=pa[8]*pb[5]+pa[9]*pb[4]+pa[10]*pb[3]+pa[11]*pb[2];
    pc[14]=pa[9]*pb[5]+pa[10]*pb[4]+pa[11]*pb[3];
    pc[15]=pa[10]*pb[5]+pa[11]*pb[4];
    pc[16]=pa[11]*pb[5];
    return;
  }
}
#endif

#endif
