/*
 * Written by Milan Straka as a part of Factoring polynomials
 * over finite fields (fapof) project.
 * Copyright (C) 2006  Milan Straka
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DECL
  char *muldata;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_CREATE
  {unsigned size=0;for (unsigned d=deg;d>0;d/=2) size+=4*sizeof(polynom_coefs)+3*d*sizeof(T);data.muldata=new char[size];}
#endif

#ifdef POLYNOMIAL_H_INCLUDING_DATA_DESTROY
  if (data.muldata) delete[] data.muldata; data.muldata=NULL;
#endif

#ifdef POLYNOMIAL_H_INCLUDING_FUNCTIONS
template<class T, int id>
inline void polynomial<T,id>::operator=(const MulWrapper& c) {
  polynom_coefs *pa=c.a.p,*pb=c.b.p;
  if (pa==p || pb==p) {
    polynom_coefs *ptmp=data.multmp;
    mul(ptmp,pa,pb);
    CLONE_COEFS(p,ptmp);
  } else mul(p,pa,pb);
}

template<class T, int id>
void polynomial<T,id>::mul(polynom_coefs *to,polynom_coefs *a,polynom_coefs *b) {
  DEGRIFY_COEFS(a);
  DEGRIFY_COEFS(b);
  if (a->deg == 0 && a->c[0]==T::fzero()) {to->deg=0;to->c[0]=T::fzero();return;}
  if (b->deg == 0 && b->c[0]==T::fzero()) {to->deg=0;to->c[0]=T::fzero();return;}
#ifndef BRUTALMUL
  if (a->deg < 4 || b->deg < 4 || a->deg * b->deg <= 25) {mul_force(to,a,b);return;}
#else
  if (a->deg < 12 && b->deg < 12 && (a->deg < 6 || b->deg < 6)) {mul_force(to,a,b);return;}
#endif

#define TMP_PUSH() char *muldata=data.muldata
#define TMP_POP() data.muldata=muldata
#define TMP_POLY(name,d) polynom_coefs *name=(polynom_coefs*)data.muldata; 	\
                         name->deg=0; name->c[0]=T::fzero();			\
                         data.muldata+=sizeof(polynom_coefs)+d*sizeof(T)

  TMP_PUSH();
  unsigned aori=a->deg,bori=b->deg;
  unsigned d=(aori > bori) ? aori/2 : bori/2;
  if (a->deg > d) a->deg=d;
  if (b->deg > d) b->deg=d;
  TMP_POLY(mya,d); TMP_POLY(myb,d);
  if (d < aori) {mya->deg=aori-d-1;for (unsigned i=d+1;i<=aori;i++) mya->c[i-d-1]=a->c[i]; }
  if (d < bori) {myb->deg=bori-d-1;for (unsigned i=d+1;i<=bori;i++) myb->c[i-d-1]=b->c[i]; }
  TMP_POLY(middle,2*d); TMP_POLY(high,2*d);
  mul(to,a,b);
  mul(high,mya,myb);
  add(mya,mya,a);
  add(myb,myb,b);
  mul(middle,mya,myb);
  sub(middle,middle,high);
  sub(middle,middle,to);
  unsigned min,addingdeg,i,j;
  //  to=to+middle << (d+1);
  addingdeg=middle->deg+d+1;  min=(to->deg < addingdeg) ? to->deg : addingdeg;
  for (i=to->deg+1;i<=d;i++) to->c[i]=T::fzero();
  for (i=d+1,j=0;i<=min;i++,j++) to->c[i]=to->c[i]+middle->c[j];
  for (;j<=middle->deg;i++,j++) to->c[i]=middle->c[j];
  if (to->deg < addingdeg) to->deg=addingdeg;
  //  to=to+high << (2d+2);
  addingdeg=high->deg+2*d+2;  min=(to->deg < addingdeg) ? to->deg : addingdeg;
  for (i=to->deg+1;i<=2*d+1;i++) to->c[i]=T::fzero();
  for (i=2*d+2,j=0;i<=min;i++,j++) to->c[i]=to->c[i]+high->c[j];
  for (;j<=high->deg;i++,j++) to->c[i]=high->c[j];
  if (to->deg < addingdeg) to->deg=addingdeg;
  DEGRIFY_COEFS(to);
  a->deg=aori;
  b->deg=bori;
  TMP_POP();  
#undef TMP_PUSH
#undef TMP_POP
#undef TMP_POLY
}

#endif
